VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DB_Sources"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private data_ As Excel.Worksheet

Public Function Init(target As Excel.Worksheet)
  Set data_ = target
End Function

Public Property Get Data() As Excel.Worksheet
  Set Data = data_
End Property

Public Property Get TargetFolder() As String
  TargetFolder = ThisWorkbook.Names(CELL_FOLDER).RefersToRange
End Property

Public Property Get FileMask() As String
  FileMask = ThisWorkbook.Names(CELL_MASK).RefersToRange
End Property

Public Property Let TargetFolder(newVal$)
  ThisWorkbook.Names(CELL_FOLDER).RefersToRange = newVal
End Property

Public Property Let FileMask(newVal$)
  ThisWorkbook.Names(CELL_MASK).RefersToRange = newVal
End Property

Public Function IBegin() As IteratorSource
  Set IBegin = New IteratorSource
  Call IBegin.Init(data_)
End Function

Public Function ILast() As IteratorSource
  Set ILast = New IteratorSource
  Call ILast.Init(data_)
  Call ILast.GoLast
End Function

Public Function Count() As Long
  Count = ILast.row_ - IBegin.row_ + 1
End Function

Public Function INew() As IteratorSource
  Set INew = New IteratorSource
  Call INew.Init(data_)
  Call INew.GoLast
  Call INew.Increment
End Function

Public Function Clear()
  Call data_.UsedRange.Offset(1).ClearContents
End Function

Public Function ClearCounters()
  Dim sHeader$: sHeader = data_.Cells(1, S_SRC_COUNTER)
  Call data_.Columns.Item(S_SRC_COUNTER).ClearContents
  data_.Cells(1, S_SRC_COUNTER) = sHeader
End Function

Public Function LoadSourcesFrom(sFolder$, sMask$) As Long
  Dim iFiles As Collection: Set iFiles = ListFilesIn(sFolder, sMask)
  LoadSourcesFrom = iFiles.Count
  
  Dim nID&: nID = Application.WorksheetFunction.Max(data_.Columns(S_SRC_ID)) + 1
  Dim iOut As IteratorSource: Set iOut = INew
  Dim sFile As Variant
  Dim fso As New Scripting.FileSystemObject
  For Each sFile In iFiles
    iOut.SourceID = nID
    iOut.FileName = fso.GetFileName(CStr(sFile))
    iOut.Extension = fso.GetExtensionName(CStr(sFile))
    iOut.Path = sFile
    nID = nID + 1
    Call iOut.Increment
  Next sFile
End Function

Public Function AcceptProcessor(iProcessor As AProcessor, Optional bShowProgress As Boolean = False)
  If bShowProgress Then
    Call CSE_ProgressBar.Init(" ", maxVal:=Count, canInterrupt:=True)
    Call CSE_ProgressBar.ShowModeless
  End If
  
  Dim iter As IteratorSource: Set iter = IBegin
  Do While Not iter.IsDone
    If bShowProgress Then
      If CSE_ProgressBar.Interrupted Then _
        Exit Do
      CSE_ProgressBar.Description = iter.FileName
    End If
    Call iProcessor.ProcessSource(iter)
    Call iter.Increment
    If bShowProgress Then _
      Call CSE_ProgressBar.IncrementA
  Loop
  
  If bShowProgress Then _
    Call Unload(CSE_ProgressBar)
End Function
