Attribute VB_Name = "Main"
Option Explicit

Public Sub RunAddSources()
  Dim iData As DB_Sources: Set iData = AccessSources
  If Not PromptFolderMask(iData) Then _
    Exit Sub
    
  Dim xlUI As New API_XLWrapper: Call xlUI.SetDocument(ThisWorkbook)
  Call xlUI.PauseUI
  Dim nCount&: nCount = iData.LoadSourcesFrom(iData.TargetFolder, iData.FileMask)
  Call xlUI.ResumeUI
  
  Call iData.Data.Activate
  Call UserInteraction.ShowMessage(IM_FILES_UPDATED, nCount)
End Sub

Public Sub RunNewSources()
  Dim iData As DB_Sources: Set iData = AccessSources
  If Not PromptFolderMask(iData) Then _
    Exit Sub
    
  Dim xlUI As New API_XLWrapper: Call xlUI.SetDocument(ThisWorkbook)
  Call xlUI.PauseUI
  Call iData.Clear
  Dim nCount&: nCount = iData.LoadSourcesFrom(iData.TargetFolder, iData.FileMask)
  Call xlUI.ResumeUI
  
  Call iData.Data.Activate
  Call UserInteraction.ShowMessage(IM_FILES_UPDATED, nCount)
End Sub

Public Sub RunClearSources()
  Call AccessSources.Clear
  Call UserInteraction.ShowMessage(IM_FILES_CLEARED)
End Sub

Public Sub RunConvertToDocx()
  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iProcessor As New ProcessorSaveAs: Call iProcessor.Init("docx", bDeleteSource:=True)
  Call iData.ClearCounters
  Call iData.AcceptProcessor(iProcessor, bShowProgress:=True)
  Call iProcessor.Finalize
  
  Call UserInteraction.ShowMessage(IM_CONVERT_EXTENSION, iProcessor.countProcessed_, iProcessor.countAll_)
End Sub

Public Sub RunMasterDocument()
  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iProcessor As New ProcessorMasterDoc: Call iProcessor.Init
  Call iData.ClearCounters
  Call iData.AcceptProcessor(iProcessor, bShowProgress:=True)
  Call iProcessor.Finalize
  
  Call iProcessor.word_.Document.ActiveWindow.Activate
  Call UserInteraction.ShowMessage(IM_MASTERDOC, iProcessor.countProcessed_, iProcessor.countAll_)
End Sub

Public Sub RunSaveAs()
  Dim sExtension$: sExtension = UserInteraction.PromptInput("  ", " ", sInitial:="pdf")
  If sExtension = vbNullString Then _
    Exit Sub
  Dim bDeleteSource As Boolean: bDeleteSource = ThisWorkbook.Names(CELL_DELETE_SOURCE).RefersToRange = 1
  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iProcessor As New ProcessorSaveAs: Call iProcessor.Init(sExtension, bDeleteSource)
  Call iData.AcceptProcessor(iProcessor, bShowProgress:=True)
  Call iProcessor.Finalize
  
  Call UserInteraction.ShowMessage(IM_CONVERT_EXTENSION, iProcessor.countProcessed_, iProcessor.countAll_)
End Sub

Public Sub RunMarkSubstitutes()
  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iSubstitutes As DB_Substitutes: Set iSubstitutes = AccesSubstitutes
  Dim iProcessor As ProcessorSubstitute: Set iProcessor = iSubstitutes.Processor
  iProcessor.GetOptions.doReplace_ = False
  iProcessor.GetOptions.mark_ = True
  
  Call iData.ClearCounters
  Call iSubstitutes.ClearCounters
  Call iData.AcceptProcessor(iProcessor, bShowProgress:=True)
  Call iProcessor.Finalize
  
  Call UserInteraction.ShowMessage(IM_SUBSTITUTES_MARKED)
End Sub

Public Sub RunExecuteSubstitutes()
  Call AccessPython.StartServer
  If Not AccessPython.Validate Then _
    Call UserInteraction.ShowMessage(EM_NO_PYTHON)
    
  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iSubstitutes As DB_Substitutes: Set iSubstitutes = AccesSubstitutes
  Dim iProcessor As ProcessorSubstitute: Set iProcessor = iSubstitutes.Processor
  iProcessor.GetOptions.doReplace_ = True
  
  Call iData.ClearCounters
  Call iSubstitutes.ClearCounters
  Call iData.AcceptProcessor(iProcessor, bShowProgress:=True)
  Call iProcessor.Finalize
  
  Call UserInteraction.ShowMessage(IM_SUBSTITUTES_DONE)
End Sub

Public Sub RunMineOrders()
  Dim iOutput As Excel.Workbook: Set iOutput = PrepareOrdersOutput()
  If iOutput Is Nothing Then _
    Exit Sub

  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iProcessor As New ProcessorOrders: Call iProcessor.Init(iOutput)
  
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim xlUI As New API_XLWrapper: Call xlUI.SetDocument(iOutput)
  Call xlUI.PauseUI

  Call iData.AcceptProcessor(iProcessor)
  Call xlUI.ResumeUI
  Call iProcessor.Finalize
  Call iOutput.Activate
  Call UserInteraction.ShowMessage(IM_ORDERS_PROCESSED, theTimer.TimeStr)
End Sub

Public Sub RunExtractUnderlined()
  Dim iOutput As Excel.Workbook: Set iOutput = PrepareFormatOutput()
  If iOutput Is Nothing Then _
    Exit Sub
  
  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iProcessor As New ProcessorFormatMine: Call iProcessor.Init(iOutput.Sheets(1), MINE_UNDERLINE)
  
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim xlUI As New API_XLWrapper: Call xlUI.SetDocument(iOutput)
  Call xlUI.PauseUI

  Call iData.AcceptProcessor(iProcessor)

  Call xlUI.ResumeUI
  Call iProcessor.Finalize
  Call iOutput.Activate
  Call UserInteraction.ShowMessage(IM_EXTRACT_SUCCESS, theTimer.TimeStr)
End Sub

Public Sub RunExtractColored()
  Dim iOutput As Excel.Workbook: Set iOutput = PrepareFormatOutput()
  If iOutput Is Nothing Then _
    Exit Sub
  
  Dim iData As DB_Sources: Set iData = AccessSources
  Dim iProcessor As New ProcessorFormatMine: Call iProcessor.Init(iOutput.Sheets(1), MINE_HIGHLIGHT)
  
   Dim theTimer As New API_Timer: Call theTimer.Start
  Dim xlUI As New API_XLWrapper: Call xlUI.SetDocument(iOutput)
  Call xlUI.PauseUI

  Call iData.AcceptProcessor(iProcessor)

  Call xlUI.ResumeUI
  Call iProcessor.Finalize
  Call iOutput.Activate
  Call UserInteraction.ShowMessage(IM_EXTRACT_SUCCESS, theTimer.TimeStr)
End Sub
