VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ProcessorOrders"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Implements AProcessor

Private word_ As API_WordWrapper
Private doc_ As Word.Document

Private sourceOut_ As IteratorSource

Private orders_ As Excel.Worksheet
Private exceptions_ As Excel.Worksheet
Private comments_ As Excel.Worksheet

Private rowOrder_ As Long
Private rowException_ As Long
Private rowComment_ As Long

Public countOrders_ As Long
Public countExceptions_ As Long
Public countComments_ As Long

Public Function Init(iOut As Excel.Workbook)
  Set word_ = New API_WordWrapper: Call word_.CreateApplication
  
  Set orders_ = iOut.Sheets(SHEET_ORDERS)
  Set exceptions_ = iOut.Sheets(SHEET_EXCEPTIONS)
  Set comments_ = iOut.Sheets(SHEET_COMMENTS)
  
  Set sourceOut_ = New IteratorSource: Call sourceOut_.Init(iOut.Sheets(SHEET_SOURCES))
  
  countOrders_ = 0
  countExceptions_ = 0
  countComments_ = 0
  
  rowOrder_ = FIRST_ROW
  rowException_ = FIRST_ROW
  rowComment_ = FIRST_ROW
End Function

Public Function Finalize()
  Call word_.ReleaseApplication
End Function

Public Function AProcessor_ProcessSource(iSource As IteratorSource)
  If ApplicationFromExtension(iSource.Extension) <> T_APP_WORD Then _
    Exit Function
  
  Set doc_ = word_.OpenDocument(iSource.Path, bReadOnly:=True)
  If doc_ Is Nothing Then _
    Exit Function
  
  Dim iOrdersRange As Word.Range: Set iOrdersRange = FindOrders()
  If iOrdersRange Is Nothing Then _
    Exit Function
  
  Dim nSource&: nSource = iSource.SourceID
  Dim nCount&: nCount = 0
  
  Dim aPar As Word.Paragraph
  For Each aPar In iOrdersRange.Paragraphs
    Dim target As Word.Range: Set target = aPar.Range
    Dim sText$: sText = SubstituteWhitespace(target.Text)
    If IsComment(target) Then
      Call AddComment(nSource, sText)
    ElseIf IsValidOrder(sText) Then
      Call ProcessOrder(sText, nSource)
      nCount = nCount + 1
    Else
      Call ProcessException(sText, nSource)
    End If
  Next aPar
  
  Call word_.ReleaseDocument(bCloseApplication:=False)
  Call OutputSource(iSource, nCount)
End Function

' ====
Private Function FindOrders() As Word.Range
  Dim iFind As Word.Range: Set iFind = doc_.Range
  With iFind.Find
    .Style = doc_.Styles(wdStyleHeading1)
    .Wrap = wdFindStop
  End With
  
  Dim nStart&: nStart = 0
  Do While iFind.Find.Execute
    If iFind.Text Like "*[]*" Then
      nStart = iFind.End
      Call iFind.Collapse(wdCollapseEnd)
      Exit Do
    End If
    Call iFind.Collapse(wdCollapseEnd)
  Loop
  If nStart = 0 Then _
    Exit Function
  If Not iFind.Find.Execute Then _
    Exit Function
    
  Dim nEnd&: nEnd = iFind.Start
  iFind.Start = nStart
  iFind.End = nEnd
  
  Set FindOrders = iFind
End Function

Private Function IsComment(target As Word.Range) As Boolean
  IsComment = True
  
  Dim sStyle$: sStyle = target.Style
  If sStyle Like "*[]*" Then _
    Exit Function
  If sStyle Like "*[] *" Then _
    Exit Function
  
  IsComment = False
End Function

Private Function IsValidOrder(sText$) As Boolean
  IsValidOrder = False
  If sText Like "*[] *" Then _
    Exit Function
  If sText Like "*[] *" Then _
    Exit Function
  If sText Like "*[][][.;:]" Then _
    Exit Function
  If sText Like "*[][][.;:]" Then _
    Exit Function
  IsValidOrder = True
End Function

Private Function ProcessOrder(sText$, nSource&) As Boolean
  Dim sPrefix$: sPrefix = ExtractPrefix(sText)
  If sPrefix <> vbNullString Then _
    sText = VBA.Trim(VBA.Right(sText, VBA.Len(sText) - VBA.Len(sPrefix) - 1))

  Dim bIsFinish As Boolean: bIsFinish = Not sText Like "*:"
  Call AddOrder(nSource, sPrefix, sText, bIsFinish)
End Function

Private Function ProcessException(sText$, nSource&)
  Dim sPrefix$: sPrefix = ExtractPrefix(sText)
  If sPrefix <> vbNullString Then _
    sText = VBA.Trim(VBA.Right(sText, VBA.Len(sText) - VBA.Len(sPrefix) - 1))

  Dim bIsFinish As Boolean: bIsFinish = Not sText Like "*:"
  Call AddException(nSource, sPrefix, sText, bIsFinish)
End Function

Private Function ExtractPrefix(sText$) As String
  Dim nPrefix&: nPrefix = 0
  Do While VBA.Mid(sText, nPrefix + 1, 1) Like "[ ().0123456789-]"
    nPrefix = nPrefix + 1
  Loop
  If nPrefix = 0 Then _
    Exit Function
  
  ExtractPrefix = VBA.Trim(VBA.Left(sText, nPrefix))
End Function

Private Function AddOrder(nSrc&, sPrefix$, sText$, bIsFinish As Boolean)
  countOrders_ = countOrders_ + 1
  
  orders_.Cells(rowOrder_, S_OUT_ID) = countOrders_
  orders_.Cells(rowOrder_, S_OUT_SRC) = nSrc
  orders_.Cells(rowOrder_, S_OUT_PREFIX) = sPrefix
  orders_.Cells(rowOrder_, S_OUT_TEXT) = sText
  orders_.Cells(rowOrder_, S_OUT_END) = IIf(bIsFinish, 1, 0)
  
  rowOrder_ = rowOrder_ + 1
End Function

Private Function AddException(nSrc&, sPrefix$, sText$, bIsFinish As Boolean)
  countExceptions_ = countExceptions_ + 1
  
  exceptions_.Cells(rowException_, S_OUT_ID) = countExceptions_
  exceptions_.Cells(rowException_, S_OUT_SRC) = nSrc
  exceptions_.Cells(rowException_, S_OUT_PREFIX) = sPrefix
  exceptions_.Cells(rowException_, S_OUT_TEXT) = sText
  exceptions_.Cells(rowException_, S_OUT_END) = IIf(bIsFinish, 1, 0)
  
  rowException_ = rowException_ + 1
End Function

Private Function AddComment(nSrc&, sText$)
  countComments_ = countComments_ + 1
  
  comments_.Cells(rowComment_, S_COM_ID) = countComments_
  comments_.Cells(rowComment_, S_COM_SRC) = nSrc
  comments_.Cells(rowComment_, S_COM_TEXT) = sText
  
  rowComment_ = rowComment_ + 1
End Function

Private Function OutputSource(iSource As IteratorSource, nCount&)
  With sourceOut_
    .Counter = nCount
    .FileName = iSource.FileName
    .Extension = iSource.Extension
    .Path = iSource.Path
    .SourceID = iSource.SourceID
  
    Call .Increment
  End With
End Function
