VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ProcessorSaveAs"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Implements AProcessor

Private targetExtension_ As String
Private deleteSource_ As Boolean

Private fso_ As Scripting.FileSystemObject
Private word_ As API_WordWrapper
Private xl_ As API_XLWrapper
Private vso_ As API_VsoWrapper

Public countProcessed_ As Long
Public countAll_ As Long

Public Function Init(sExtension$, bDeleteSource As Boolean)
  Set fso_ = New Scripting.FileSystemObject
  countProcessed_ = 0
  countAll_ = 0
  targetExtension_ = sExtension
  deleteSource_ = bDeleteSource
End Function

Public Function Finalize()
  If Not word_ Is Nothing Then _
    Call word_.ReleaseApplication
  If Not xl_ Is Nothing Then _
    Call xl_.ReleaseApplication
  If Not vso_ Is Nothing Then _
    Call vso_.ReleaseApplication
End Function

Public Function AProcessor_ProcessSource(iSource As IteratorSource)
  Dim sExt$: sExt = VBA.UCase(iSource.Extension)
  If sExt = VBA.UCase(targetExtension_) Then
    countAll_ = countAll_ + 1
    iSource.Counter = 0
    Exit Function
  End If
  
  Dim iWrapper As Object: Set iWrapper = GetWrapper(ApplicationFromExtension(sExt))
  If iWrapper Is Nothing Then _
    Exit Function
    
  countAll_ = countAll_ + 1
  iSource.Counter = -1
  
  Dim sOldPath$: sOldPath = iSource.Path
  If iWrapper.OpenDocument(sOldPath, bReadOnly:=False) Is Nothing Then _
    Exit Function
  
  Dim sNewPath$: sNewPath = fso_.GetParentFolderName(sOldPath) & "\" & fso_.GetBaseName(sOldPath) & "." & targetExtension_
  If fso_.FileExists(sNewPath) Then _
    GoTo SAFE_EXIT
  If Not iWrapper.SaveAs(sNewPath) Then _
    GoTo SAFE_EXIT
  
  countProcessed_ = countProcessed_ + 1
  iSource.Counter = 1
  iSource.FileName = fso_.GetFileName(sNewPath)
  iSource.Extension = "docx"
  iSource.Path = sNewPath
  
  Call word_.ReleaseDocument(bCloseApplication:=False)
  If deleteSource_ Then _
    Call fso_.DeleteFile(sOldPath)
  Exit Function
  
SAFE_EXIT:
  Call word_.ReleaseDocument(bCloseApplication:=False)
End Function

' ======
Private Function GetWrapper(nAppl As TApplication) As Object
  Select Case nAppl
    Case T_APP_WORD
      If word_ Is Nothing Then
        Set word_ = New API_WordWrapper
        Call word_.CreateApplication
      End If
      Set GetWrapper = word_
      
    Case T_APP_EXCEL
      If xl_ Is Nothing Then
        Set xl_ = New API_XLWrapper
        Call xl_.CreateApplication
      End If
      Set GetWrapper = xl_
      
    Case T_APP_VISIO
    If vso_ Is Nothing Then
      Set vso_ = New API_VsoWrapper
      Call vso_.CreateApplication
    End If
    Set GetWrapper = vso_
  End Select
End Function
