VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ProcessorSubstitute"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Implements AProcessor

Private substitutes_ As DB_Substitutes
Private options_ As OptionsSubstitutes

Private word_ As API_WordWrapper
Private visio_ As API_VsoWrapper
Private hasPython_ As Boolean

Public Function Init(iOptions As OptionsSubstitutes, iSubstitutes As DB_Substitutes)
  Set options_ = iOptions
  Set substitutes_ = iSubstitutes
  hasPython_ = AccessPython.Validate
End Function

Public Function Finalize()
  If Not word_ Is Nothing Then _
    Call word_.ReleaseApplication
  If Not visio_ Is Nothing Then _
    Call visio_.ReleaseApplication
End Function

Public Function GetOptions() As OptionsSubstitutes
  Set GetOptions = options_
End Function

Public Function AProcessor_ProcessSource(iSource As IteratorSource)
  Dim iAppl As TApplication: iAppl = ApplicationFromExtension(iSource.Extension)
  If iAppl = T_APP_VISIO Then
    Call ProcessVisio(iSource)
  ElseIf iAppl = T_APP_WORD Then
    Call ProcessWord(iSource)
  End If
End Function

' ======
Private Function AccessWord() As API_WordWrapper
  If word_ Is Nothing Then
    Set word_ = New API_WordWrapper
    Call word_.CreateApplication
  End If
  Set AccessWord = word_
End Function

Private Function AccessVisio() As API_VsoWrapper
  If visio_ Is Nothing Then
    Set visio_ = New API_VsoWrapper
    Call visio_.CreateApplication
  End If
  Set AccessVisio = visio_
End Function

Private Function ProcessVisio(iSource As IteratorSource)
  Dim iDoc As Visio.Document: Set iDoc = AccessVisio.OpenDocument(iSource.Path)
  If iDoc Is Nothing Then _
    Exit Function
  Dim iShape As Visio.Shape
  Dim iPage As Visio.Page
  For Each iPage In iDoc.Pages
    For Each iShape In iPage.Shapes
      If iShape.Type = visTypeShape Then _
         Call iSource.IncrementCounter(ProcessShapeText(iShape))
    Next iShape
  Next iPage
  
  Call AccessVisio.ReleaseDocument(bCloseApplication:=False, bSaveChanges:=True)
End Function

Private Function ProcessShapeText(target As Visio.Shape) As Long
  Dim shpText$: shpText = VBA.Trim(target.Text)
  If shpText = vbNullString Then _
    Exit Function

  Dim iter As IteratorSubstitute: Set iter = substitutes_.IBegin
  Do While Not iter.IsDone
    Dim sNeedle$: sNeedle = iter.FindWhat
    If Not RegexTest(shpText, sNeedle) Then _
      GoTo NEXT_TEST
    
    ProcessShapeText = ProcessShapeText + 1
    Call iter.IncrementCounter
    If options_.doReplace_ Then _
      target.Text = RegexReplace(shpText, sNeedle, iter.Replacement, 1)
    If options_.mark_ Then _
      target.Characters.CharProps(visCharacterStyle) = visItalic

NEXT_TEST:
    Call iter.Increment
  Loop
End Function

Private Function ProcessWord(iSource As IteratorSource)
  Dim iDoc As Word.Document: Set iDoc = AccessWord.OpenDocument(iSource.Path, bReadOnly:=False)
  If iDoc Is Nothing Then _
    Exit Function
    
  If CSE_ProgressBar.Visible Then _
    Call CSE_ProgressBar.InitSecondBar(maxVal:=substitutes_.Count)
    
  Dim bNeedSpaces As Boolean
  Dim iter As IteratorSubstitute: Set iter = substitutes_.IBegin
  Do While Not iter.IsDone
    Dim rFind As Word.Range: Set rFind = iDoc.Range
    bNeedSpaces = iter.IsMultipleWords And options_.wholeWord_
    
    If CSE_ProgressBar.Visible Then _
      If CSE_ProgressBar.Interrupted Then _
        Exit Do
    
    With rFind.Find
      .Text = IIf(bNeedSpaces, PrepareSearchText(iter.FindWhat), iter.FindWhat)
      .MatchCase = options_.caseSensitive_
      .MatchWholeWord = options_.wholeWord_
      .MatchWildcards = options_.regex_ Or bNeedSpaces
    End With

    Do While rFind.Find.Execute
      If bNeedSpaces Then
        rFind.Start = rFind.Start + 1
        rFind.End = rFind.End - 1
      End If
      
      If options_.doReplace_ Then _
        Call ReplaceTextInRange(rFind, iter.Replacement, iter.UseMorpho)
    
      Call iSource.IncrementCounter
      Call iter.IncrementCounter
      
      If options_.mark_ Then _
        rFind.HighlightColorIndex = IIf(options_.doReplace_, wdGray25, wdGray50)
      Call rFind.Collapse(wdCollapseEnd)
    Loop
    
    If CSE_ProgressBar.Visible Then _
      Call CSE_ProgressBar.IncrementB
    Call iter.Increment
  Loop
  
  If CSE_ProgressBar.Visible Then _
    Call CSE_ProgressBar.HideSecondBar
  
  Call AccessWord.ReleaseDocument(bCloseApplication:=False, bSaveChanges:=True)
End Function

Private Function PrepareSearchText(sQuery$) As String
  Dim sFirstSymbol$: sFirstSymbol = VBA.Left$(sQuery, 1)
  If sFirstSymbol <> "[" Then
    PrepareSearchText = "[" & VBA.UCase(sFirstSymbol) & VBA.LCase(sFirstSymbol) & "]" & VBA.Right(sQuery, VBA.Len(sQuery) - 1)
  Else
    PrepareSearchText = sQuery
  End If
  PrepareSearchText = "[ ^0013^0009\-]" & PrepareSearchText & "[ ^0013^0009.,\!\?\-\)]"
End Function

Private Function ReplaceTextInRange(target As Word.Range, sReplacement$, bMorpho As Boolean)
  If bMorpho And hasPython_ Then
    Dim sNextWord$, sPrevWord$
    If target.Next(wdCharacter, 1) = " " Then
      Dim nextWordRange As Word.Range: Set nextWordRange = target.Duplicate
      Call nextWordRange.Collapse(wdCollapseEnd)
      Set nextWordRange = nextWordRange.Next(wdWord, 1)
      sNextWord = VBA.Trim(nextWordRange.Text)
      If Not IsAlphaNumeric(sNextWord) Then sNextWord = ""
    End If
    
    If target.Previous(wdCharacter, 1) = " " Then
      Dim prevWordRange As Word.Range: Set prevWordRange = target.Duplicate
      Call prevWordRange.Collapse(wdCollapseStart)
      Set prevWordRange = prevWordRange.Previous(wdWord, 1)
      sPrevWord = VBA.Trim(prevWordRange.Text)
      If Not IsAlphaNumeric(sPrevWord) Then sPrevWord = ""
    End If
    
    sReplacement = ApplyMorpho(sReplacement, sPrevWord, sNextWord)
  End If
  
  Dim sFirstSymbol$: sFirstSymbol = VBA.Left$(target.Text, 1)
  If sFirstSymbol Like "[-A-Z]" Then
    target.Text = CapitalizeFirstLetter(sReplacement)
  Else
    target.Text = sReplacement
  End If
End Function

Private Function ApplyMorpho(sTarget$, sPrev$, sNext$) As String
  Dim sInflected$: sInflected = AccessPython.CallFunction(PY_MODULE_TEXT, "inflect_context", Array(sTarget, sPrev, sNext))
  If sInflected = "" Or sInflected Like "*error*" Then _
    sInflected = sTarget
    
  ApplyMorpho = sInflected
End Function

Private Function ExtractAdjectiveForm(sParse$) As String
  Dim sResult$
  If VBA.InStr(1, sParse, "plur") <> 0 Then
    sResult = "plur"
  Else
    If VBA.InStr(1, sParse, "femn") <> 0 Then
      sResult = "femn,sing"
    ElseIf VBA.InStr(1, sParse, "masc") <> 0 Then
      sResult = "masc,sing"
    Else
      sResult = "neut,sing"
    End If
  End If
  sResult = sResult & "," & VBA.Right(sParse, VBA.Len(sParse) - VBA.InStrRev(sParse, ","))
  ExtractAdjectiveForm = sResult
End Function
